@@ -8,9 +8,9 @@
 #ifndef CUSTOM_H
 #define	CUSTOM_H
 
-#define LINSSIDVERSION "2.7"
-#define LINSSIDPREFSVER "2.3"
-#define LINSSIDDATALOGVER "2.3"
+#define LINSSIDVERSION "2.9"
+#define LINSSIDPREFSVER "2.8"
+#define LINSSIDDATALOGVER "2.8"
 #define DATAWANTED QEvent::User + 1
 #define DATAREADY QEvent::User + 2
 #define NUMBER_OF_COLORS 13
@@ -21,14 +21,6 @@
 #define PREFS_FILE_NAME ".linssid.prefs"
 #define LOG_DATA_FILE_NAME "LinSSID.datalog"
 
-enum wlPrivacy {
-    none, WEP, WPA2
-};
-
-enum wlCipher {
-    unknown, TKIP, AES
-};
-
 enum passStatus {
     GOOD, BAD, CANCELLED, NOTNEEDED
 };
@@ -41,9 +33,14 @@ enum wlDriver {
 // and the tableHeader variable of the Qwt table
 enum colTitle {
     PLOT, SSID, MAC, CHANNEL, MODE, SECURITY, PRIVACY,
-    CIPHER, FREQUENCY, QUALITY, SIGNAL, NOISE, MINSIGNAL, MAXSIGNAL, MBPS,
+    CIPHER, FREQUENCY, QUALITY, SIGNAL, BW, MINSIGNAL, MAXSIGNAL, CENCHAN,
     FIRST_SEEN, LAST_SEEN, VENDOR, PROTOCOL, TYPE // TYPE not yet impl
 };
 
+enum pageBlockType {
+    BT_NONE, BT_BSS, BT_RSN, BT_BSS_LOAD, BT_HT_CAPABILITIES, BT_HT_OPERATION, BT_EXTENDED_CAPABILITIES,
+    BT_VHT_CAPABILITIES, BT_VHT_OPERATION, BT_WPA, BT_WPS, BT_WMM
+};
+
 
 #endif	/* CUSTOM_H */
