@@ -10,6 +10,7 @@
 #include <string>
 #include <sstream>
 #include <random>
+#include <climits>
 #include <QtCore>
 #include <QString>
 #include <QThread>
@@ -59,17 +60,17 @@ struct MainForm::cellData {
     string essid;
     string mode; // master, managed, etc.
     string security; // on or off
-    wlPrivacy privacy; // none or WEP or WPA2
-    wlCipher cipher; // unknown or TKIP or AES
+    string privacy; // group cipher
+    string cipher; // pairwise cipher
     int channel;
     string frequency;
-    string protocol; // a,b,g,n
-    int quality; // normalized: x/y * 100
+    string protocol; // b,g,n,ac
+    int quality; // per microsoft definition, derived from signal
     int signal; // try to get dBm
     int minSignal; // lowest seen
     int maxSignal; // highest seen
-    int noise; // try to get dBm
-    int mbps; // try to get Mb/s
+    int BW; // max BW in any protocol 20 or 40 or 80 or 160 MHz
+    int cenChan; // center channel in 40/80/160 MHz bandwidths
     string vendor;
     long firstSeen;
     long lastSeen;
@@ -136,6 +137,8 @@ struct MainForm::sDefPref {
     int logData;
 };
 
+
+
 // declare some variables
 Getter* MainForm::pGetter; // a pointer to our data getter
 QThread* MainForm::pGetterThread; // a pointer to its thread
@@ -147,18 +150,11 @@ int MainForm::numVendors;
 int MainForm::maxVendorRecL;
 long MainForm::runStartTime;
 long MainForm::now; // absolute time of the block
+pageBlockType pageBlock; // which section of page is data coming from
 int MainForm::logDataState;
 long MainForm::blockSampleTime; // time of the block relative to runStartTime
 bool MainForm::firstScan; // do we need to get sudo privileges?
-wlPrivacy MainForm::currentPrivacy;
-string MainForm::wlPrivacies[3]
-{
-    "none", "WEP", "WPA2"
-};
-string MainForm::wlCiphers[3]
-{
-    "unknown", "TKIP", "AES"
-};
+
 string MainForm::chan24Freq[15] {
     "0.000","2.412","2.417","2.422","2.427",
     "2.432","2.437","2.442","2.447","2.452",
@@ -233,10 +229,10 @@ MainForm::MainForm() {
     connect(MainForm::mainFormWidget.actionFrequency, SIGNAL(changed()), this, SLOT(reDrawTable()));
     connect(MainForm::mainFormWidget.actionQuality, SIGNAL(changed()), this, SLOT(reDrawTable()));
     connect(MainForm::mainFormWidget.actionSignal, SIGNAL(changed()), this, SLOT(reDrawTable()));
-    connect(MainForm::mainFormWidget.actionNoise, SIGNAL(changed()), this, SLOT(reDrawTable()));
+    connect(MainForm::mainFormWidget.actionBW, SIGNAL(changed()), this, SLOT(reDrawTable()));
     connect(MainForm::mainFormWidget.actionMin_Signal, SIGNAL(changed()), this, SLOT(reDrawTable()));
     connect(MainForm::mainFormWidget.actionMax_Signal, SIGNAL(changed()), this, SLOT(reDrawTable()));
-    connect(MainForm::mainFormWidget.actionMbps, SIGNAL(changed()), this, SLOT(reDrawTable()));
+    connect(MainForm::mainFormWidget.actionCenChan, SIGNAL(changed()), this, SLOT(reDrawTable()));
     connect(MainForm::mainFormWidget.actionFirst_Seen, SIGNAL(changed()), this, SLOT(reDrawTable()));
     connect(MainForm::mainFormWidget.actionLast_Seen, SIGNAL(changed()), this, SLOT(reDrawTable()));
     connect(MainForm::mainFormWidget.actionVendor, SIGNAL(changed()), this, SLOT(reDrawTable()));
@@ -290,7 +286,7 @@ void MainForm::initColtoAction() {
     // brutal -- set the array of pointers from columns to their menu items
     // these should have the same elements as the enum in 'custom.h'
     // PLOT, SSID, MAC, CHANNEL, MODE, SECURITY, PRIVACY,
-    // CIPHER, FREQUENCY, QUALITY, SIGNAL, NOISE, MINSIGNAL, MAXSIGNAL, MBPS,
+    // CIPHER, FREQUENCY, QUALITY, SIGNAL, BW, MINSIGNAL, MAXSIGNAL, CENCHAN,
     // FIRST_SEEN, LAST_SEEN, VENDOR, PROTOCOL
     MainForm::colToQAction[PLOT] = MainForm::mainFormWidget.actionPlot;
     MainForm::colToQAction[SSID] = MainForm::mainFormWidget.actionSSID;
@@ -304,10 +300,10 @@ void MainForm::initColtoAction() {
     MainForm::colToQAction[FREQUENCY] = MainForm::mainFormWidget.actionFrequency;
     MainForm::colToQAction[QUALITY] = MainForm::mainFormWidget.actionQuality;
     MainForm::colToQAction[SIGNAL] = MainForm::mainFormWidget.actionSignal;
-    MainForm::colToQAction[NOISE] = MainForm::mainFormWidget.actionNoise;
+    MainForm::colToQAction[BW] = MainForm::mainFormWidget.actionBW;
     MainForm::colToQAction[MINSIGNAL] = MainForm::mainFormWidget.actionMin_Signal;
     MainForm::colToQAction[MAXSIGNAL] = MainForm::mainFormWidget.actionMax_Signal;
-    MainForm::colToQAction[MBPS] = MainForm::mainFormWidget.actionMbps;
+    MainForm::colToQAction[CENCHAN] = MainForm::mainFormWidget.actionCenChan;
     MainForm::colToQAction[FIRST_SEEN] = MainForm::mainFormWidget.actionFirst_Seen;
     MainForm::colToQAction[LAST_SEEN] = MainForm::mainFormWidget.actionLast_Seen;
     MainForm::colToQAction[VENDOR] = MainForm::mainFormWidget.actionVendor;
@@ -814,12 +810,12 @@ void MainForm::drawTable() {
     /*
     enum colTitle {
     PLOT, SSID, MAC, CHANNEL, MODE, SECURITY, PRIVACY,
-    CIPHER, FREQUENCY, QUALITY, SIGNAL, NOISE, MINSIGNAL, MAXSIGNAL, MBPS,
+    CIPHER, FREQUENCY, QUALITY, SIGNAL, BW, MINSIGNAL, MAXSIGNAL, CENCHAN,
     FIRST_SEEN, LAST_SEEN, VENDOR, PROTOCOL // TYPE not yet impl
 }; */
     MainForm::mainFormWidget.mainTableWidget->setHorizontalHeaderLabels(
             QString("Plot|SSID|MAC|Channel|Mode|Security|Privacy|Cipher|Frequency\
-|Quality|Signal|Noise|Min Sig|Max Sig|Mbps|First Seen|Last Seen|Vendor|Protocol|Type").split("|"));
+|Quality|Signal|BW MHz|Min Sig|Max Sig|Cen Chan|First Seen|Last Seen|Vendor|Protocol|Type").split("|"));
     setVisibleCols();
     MainForm::mainFormWidget.mainTableWidget->horizontalHeader()->setSectionsMovable(true);
     MainForm::mainFormWidget.mainTableWidget->horizontalHeader()
@@ -851,14 +847,14 @@ void MainForm::setVisibleCols() {
             !(MainForm::mainFormWidget.actionQuality->isChecked()));
     MainForm::mainFormWidget.mainTableWidget->setColumnHidden(SIGNAL,
             !(MainForm::mainFormWidget.actionSignal->isChecked()));
-    MainForm::mainFormWidget.mainTableWidget->setColumnHidden(NOISE,
-            !(MainForm::mainFormWidget.actionNoise->isChecked()));
+    MainForm::mainFormWidget.mainTableWidget->setColumnHidden(BW,
+            !(MainForm::mainFormWidget.actionBW->isChecked()));
     MainForm::mainFormWidget.mainTableWidget->setColumnHidden(MINSIGNAL,
             !(MainForm::mainFormWidget.actionMin_Signal->isChecked()));
     MainForm::mainFormWidget.mainTableWidget->setColumnHidden(MAXSIGNAL,
             !(MainForm::mainFormWidget.actionMax_Signal->isChecked()));
-    MainForm::mainFormWidget.mainTableWidget->setColumnHidden(MBPS,
-            !(MainForm::mainFormWidget.actionMbps->isChecked()));
+    MainForm::mainFormWidget.mainTableWidget->setColumnHidden(CENCHAN,
+            !(MainForm::mainFormWidget.actionCenChan->isChecked()));
     MainForm::mainFormWidget.mainTableWidget->setColumnHidden(FIRST_SEEN,
             !(MainForm::mainFormWidget.actionFirst_Seen->isChecked()));
     MainForm::mainFormWidget.mainTableWidget->setColumnHidden(LAST_SEEN,
@@ -871,57 +867,57 @@ void MainForm::fillTable() {
     // fill in the x-y, also set each cell text alignment
     MainForm::mainFormWidget.mainTableWidget->setRowCount(maxTableIndex + 1);
     for (int row = 0; row <= maxTableIndex; row++) {
-        MainForm::cellDataRay[row].pTableItem[PLOT]->setTextAlignment(Qt::AlignHCenter);
+        MainForm::cellDataRay[row].pTableItem[PLOT]->setTextAlignment(Qt::AlignCenter);
         MainForm::cellDataRay[row].pTableItem[SSID]->
                 setText(MainForm::cellDataRay[row].essid.c_str());
         MainForm::cellDataRay[row].pTableItem[MAC]->
                 setText(MainForm::cellDataRay[row].macAddr.c_str());
-        MainForm::cellDataRay[row].pTableItem[MAC]->setTextAlignment(Qt::AlignHCenter);
+        MainForm::cellDataRay[row].pTableItem[MAC]->setTextAlignment(Qt::AlignCenter);
         MainForm::cellDataRay[row].pTableItem[CHANNEL]->
                 setData(Qt::DisplayRole, MainForm::cellDataRay[row].channel);
-        MainForm::cellDataRay[row].pTableItem[CHANNEL]->setTextAlignment(Qt::AlignHCenter);
+        MainForm::cellDataRay[row].pTableItem[CHANNEL]->setTextAlignment(Qt::AlignCenter);
         MainForm::cellDataRay[row].pTableItem[MODE]->
                 setText(MainForm::cellDataRay[row].mode.c_str());
-        MainForm::cellDataRay[row].pTableItem[MODE]->setTextAlignment(Qt::AlignHCenter);
+        MainForm::cellDataRay[row].pTableItem[MODE]->setTextAlignment(Qt::AlignCenter);
         MainForm::cellDataRay[row].pTableItem[PROTOCOL]->
                 setText(MainForm::cellDataRay[row].protocol.c_str());
-        MainForm::cellDataRay[row].pTableItem[PROTOCOL]->setTextAlignment(Qt::AlignHCenter);
+        MainForm::cellDataRay[row].pTableItem[PROTOCOL]->setTextAlignment(Qt::AlignCenter);
         MainForm::cellDataRay[row].pTableItem[SECURITY]->
                 setText(MainForm::cellDataRay[row].security.c_str());
-        MainForm::cellDataRay[row].pTableItem[SECURITY]->setTextAlignment(Qt::AlignHCenter);
+        MainForm::cellDataRay[row].pTableItem[SECURITY]->setTextAlignment(Qt::AlignCenter);
         MainForm::cellDataRay[row].pTableItem[PRIVACY]->
-                setText((MainForm::wlPrivacies[MainForm::cellDataRay[row].privacy]).c_str());
-        MainForm::cellDataRay[row].pTableItem[PRIVACY]->setTextAlignment(Qt::AlignHCenter);
+                setText((MainForm::cellDataRay[row].privacy).c_str());
+        MainForm::cellDataRay[row].pTableItem[PRIVACY]->setTextAlignment(Qt::AlignCenter);
         MainForm::cellDataRay[row].pTableItem[CIPHER]->
-                setText((MainForm::wlCiphers[MainForm::cellDataRay[row].cipher]).c_str());
-        MainForm::cellDataRay[row].pTableItem[CIPHER]->setTextAlignment(Qt::AlignHCenter);
+                setText((MainForm::cellDataRay[row].cipher).c_str());
+        MainForm::cellDataRay[row].pTableItem[CIPHER]->setTextAlignment(Qt::AlignCenter);
         MainForm::cellDataRay[row].pTableItem[FREQUENCY]->
                 setText(MainForm::cellDataRay[row].frequency.c_str());
-        MainForm::cellDataRay[row].pTableItem[FREQUENCY]->setTextAlignment(Qt::AlignHCenter);
+        MainForm::cellDataRay[row].pTableItem[FREQUENCY]->setTextAlignment(Qt::AlignCenter);
         MainForm::cellDataRay[row].pTableItem[QUALITY]->
                 setData(Qt::DisplayRole, MainForm::cellDataRay[row].quality);
-        MainForm::cellDataRay[row].pTableItem[QUALITY]->setTextAlignment(Qt::AlignHCenter);
+        MainForm::cellDataRay[row].pTableItem[QUALITY]->setTextAlignment(Qt::AlignCenter);
         MainForm::cellDataRay[row].pTableItem[SIGNAL]->
                 setData(Qt::DisplayRole, MainForm::cellDataRay[row].signal);
-        MainForm::cellDataRay[row].pTableItem[SIGNAL]->setTextAlignment(Qt::AlignHCenter);
-        MainForm::cellDataRay[row].pTableItem[NOISE]->
-                setData(Qt::DisplayRole, MainForm::cellDataRay[row].noise);
-        MainForm::cellDataRay[row].pTableItem[NOISE]->setTextAlignment(Qt::AlignHCenter);
+        MainForm::cellDataRay[row].pTableItem[SIGNAL]->setTextAlignment(Qt::AlignCenter);
+        MainForm::cellDataRay[row].pTableItem[BW]->
+                setData(Qt::DisplayRole, MainForm::cellDataRay[row].BW);
+        MainForm::cellDataRay[row].pTableItem[BW]->setTextAlignment(Qt::AlignCenter);
         MainForm::cellDataRay[row].pTableItem[MINSIGNAL]->
                 setData(Qt::DisplayRole, MainForm::cellDataRay[row].minSignal);
-        MainForm::cellDataRay[row].pTableItem[MINSIGNAL]->setTextAlignment(Qt::AlignHCenter);
+        MainForm::cellDataRay[row].pTableItem[MINSIGNAL]->setTextAlignment(Qt::AlignCenter);
         MainForm::cellDataRay[row].pTableItem[MAXSIGNAL]->
                 setData(Qt::DisplayRole, MainForm::cellDataRay[row].maxSignal);
-        MainForm::cellDataRay[row].pTableItem[MAXSIGNAL]->setTextAlignment(Qt::AlignHCenter);
-        MainForm::cellDataRay[row].pTableItem[MBPS]->
-                setData(Qt::DisplayRole, MainForm::cellDataRay[row].mbps);
-        MainForm::cellDataRay[row].pTableItem[MBPS]->setTextAlignment(Qt::AlignHCenter);
+        MainForm::cellDataRay[row].pTableItem[MAXSIGNAL]->setTextAlignment(Qt::AlignCenter);
+        MainForm::cellDataRay[row].pTableItem[CENCHAN]->
+                setData(Qt::DisplayRole, MainForm::cellDataRay[row].cenChan);
+        MainForm::cellDataRay[row].pTableItem[CENCHAN]->setTextAlignment(Qt::AlignCenter);
         MainForm::cellDataRay[row].pTableItem[FIRST_SEEN]->
                 setText(QDateTime::fromTime_t(MainForm::cellDataRay[row].firstSeen).toString("MM/dd-hh:mm:ss"));
-        MainForm::cellDataRay[row].pTableItem[FIRST_SEEN]->setTextAlignment(Qt::AlignHCenter);
+        MainForm::cellDataRay[row].pTableItem[FIRST_SEEN]->setTextAlignment(Qt::AlignCenter);
         MainForm::cellDataRay[row].pTableItem[LAST_SEEN]->
                 setText(QDateTime::fromTime_t(MainForm::cellDataRay[row].lastSeen).toString("MM/dd-hh:mm:ss"));
-        MainForm::cellDataRay[row].pTableItem[LAST_SEEN]->setTextAlignment(Qt::AlignHCenter);
+        MainForm::cellDataRay[row].pTableItem[LAST_SEEN]->setTextAlignment(Qt::AlignCenter);
         MainForm::cellDataRay[row].pTableItem[VENDOR]->
                 setText(MainForm::cellDataRay[row].vendor.c_str());
     }
@@ -990,10 +986,11 @@ void MainForm::fillPlots() {
             MainForm::blockSampleTime, 10);
     for (int tbi = 0; tbi <= maxTableIndex; tbi++) {
         // first the 2.5GHz and 5GHz channel vs signal plots
-        MainForm::cellDataRay[tbi].xPlot[0] = MainForm::cellDataRay[tbi].channel - 2.0;
-        MainForm::cellDataRay[tbi].xPlot[1] = MainForm::cellDataRay[tbi].channel - 1.0;
-        MainForm::cellDataRay[tbi].xPlot[2] = MainForm::cellDataRay[tbi].channel + 1.0;
-        MainForm::cellDataRay[tbi].xPlot[3] = MainForm::cellDataRay[tbi].channel + 2.0;
+        float spread = MainForm::cellDataRay[tbi].BW / 10.0;
+        MainForm::cellDataRay[tbi].xPlot[0] = MainForm::cellDataRay[tbi].cenChan - spread;
+        MainForm::cellDataRay[tbi].xPlot[1] = MainForm::cellDataRay[tbi].cenChan - spread + 1.0;
+        MainForm::cellDataRay[tbi].xPlot[2] = MainForm::cellDataRay[tbi].cenChan + spread - 1.0;
+        MainForm::cellDataRay[tbi].xPlot[3] = MainForm::cellDataRay[tbi].cenChan + spread;
         MainForm::cellDataRay[tbi].yPlot[0] = MainForm::cellDataRay[tbi].yPlot[3] = -100.0;
         MainForm::cellDataRay[tbi].yPlot[1] = MainForm::cellDataRay[tbi].yPlot[2]
                 = MainForm::cellDataRay[tbi].signal;
@@ -1030,11 +1027,12 @@ void MainForm::fillPlots() {
 
 void MainForm::initNewCell(string macAddress, int tbi) {
     // Initialize a newly found cell.
-    MainForm::cellData* emptyStruct; // C++ is really stupid
-    emptyStruct = new MainForm::cellData(); // unintuitive as heck
-    MainForm::cellDataRay.push_back(*emptyStruct); // This is the only way
-    delete emptyStruct; // to create a new initialized element
+    MainForm::cellData* emptyStruct;               // C++ is really stupid
+    emptyStruct = new MainForm::cellData();        // unintuitive as heck
+    MainForm::cellDataRay.push_back(*emptyStruct); // This is the only way to
+    delete emptyStruct;                            // create a new initialized element
     MainForm::cellDataRay[tbi].macAddr = macAddress; // insert MAC address
+    MainForm::cellDataRay[tbi].essid = "<hidden>"; // nl80211 iw doesn't report SSID line if hidden
     MainForm::cellDataRay[tbi].minSignal = 0;
     MainForm::cellDataRay[tbi].maxSignal = -120;
     MainForm::cellDataRay[tbi].firstSeen = now;
@@ -1047,6 +1045,10 @@ void MainForm::initNewCell(string macAddress, int tbi) {
     MainForm::cellDataRay[tbi].pTimeCurve->setPen(* new QPen(tempColor, 3.0));
     MainForm::cellDataRay[tbi].pTimeCurve->setRenderHint(QwtPlotItem::RenderAntialiased);
     MainForm::cellDataRay[tbi].pTimeCurve->attach(MainForm::mainFormWidget.timePlot);
+    MainForm::cellDataRay[tbi].pBandCurve = new QwtPlotCurve("");
+    MainForm::cellDataRay[tbi].pBandCurve->setPen(* new QPen(MainForm::cellDataRay[tbi].color, 3.0));
+    MainForm::cellDataRay[tbi].pBandCurve->setRenderHint(QwtPlotItem::RenderAntialiased);
+    // attaching plot curve waits 'till know frequency
     MainForm::mainFormWidget.mainTableWidget->setRowCount(tbi + 1);
     for (int ix = 0; ix < MAX_TABLE_COLS; ix++) {
         MainForm::cellDataRay[tbi].pTableItem[ix] = new QTableWidgetItem(); // Give it a table item for each column
@@ -1061,14 +1063,20 @@ void MainForm::initNewCell(string macAddress, int tbi) {
 
 void MainForm::extractData(string tl, int &tbi) {
     // extract the information from each line recovered from the pipe from getter
+    static int newBSS;
     boost::smatch sm;
-    if (boost::regex_match(tl, boost::regex(".*?Cell [0-9][0-9].*"))) {
-        string macAddress = boost::regex_replace(tl,
-                boost::regex(".+?((?:[A-Fa-f0-9]{2}:){5}[A-Fa-f0-9]{2}).*"), "$1");
+    if (boost::regex_match(tl, sm, boost::regex("^BSS.*?(([A-Fa-f0-9]{2}:){5}[A-Fa-f0-9]{2}).*",
+          boost::regex_constants::icase))) {
+        pageBlock = BT_BSS;
+        string macAddress = sm[1];
+        //string macAddress = boost::regex_replace(tl,
+        //        boost::regex(".+?((?:[A-Fa-f0-9]{2}:){5}[A-Fa-f0-9]{2}).*"), "$1");
+        for (unsigned int ic=0; ic< macAddress.length(); ic++) macAddress[ic] = toupper(macAddress[ic]);
         tbi = MainForm::maxTableIndex + 1;
         for (int i = 0; i <= MainForm::maxTableIndex; i++) {
             if (macAddress == MainForm::cellDataRay[i].macAddr) {
                 tbi = i;
+                newBSS = 0; // Have seen this BSS before
                 break;
             }
         }
@@ -1076,158 +1084,125 @@ void MainForm::extractData(string tl, int &tbi) {
             // at this point we have found a **new** mac address. Initialize accordingly.
             MainForm::initNewCell(macAddress, tbi);
             MainForm::maxTableIndex = tbi;
+            newBSS = 1; // Flag first time through for new BSS
         }
         MainForm::cellDataRay[tbi].timesSeen++;
         MainForm::cellDataRay[tbi].lastSeen = now;
-    } else if (boost::regex_match(tl, sm, boost::regex(".*ESSID *: *\"(.*)\".*"))) {
+        MainForm::cellDataRay[tbi].BW = 20; // all have at least 20 MHz bandwidth
+        MainForm::cellDataRay[tbi].protocol = "";
+    } 
+       else if (boost::regex_match(tl, sm, boost::regex("^.+?SSID: +(.*)"))) {
         string tempSSID = sm[1];
-        if (tempSSID.length() == 0) tempSSID = "<hidden>";
-        if (tempSSID.substr(0, 4) == "\\x00") tempSSID = "<hidden>";
-        if (MainForm::cellDataRay[tbi].essid.length() == 0
-                || MainForm::cellDataRay[tbi].essid == "<hidden>")
-            MainForm::cellDataRay[tbi].essid = tempSSID;
-    } else if (boost::regex_match(tl, sm, boost::regex(
-            " *Protocol *:.*802\\.11 *([^ ]*) *",
+        MainForm::cellDataRay[tbi].essid = tempSSID;
+    }  else if (boost::regex_match(tl, sm, boost::regex(
+            "^[ \\t]+Supported rates: (.*)", boost::regex_constants::icase))) { // protocol
+        string tempStr = sm[1];
+        if (MainForm::MinIntStr(tempStr) < 11) MainForm::cellDataRay[tbi].protocol += "b";
+        if (MainForm::MaxIntStr(tempStr) >= 11) MainForm::cellDataRay[tbi].protocol += "g";
+    }  else if (boost::regex_match(tl, sm, boost::regex(
+            "^[ \\t]+HT Capabilities:", boost::regex_constants::icase))) { // protocol
+        pageBlock = BT_HT_CAPABILITIES;
+        MainForm::cellDataRay[tbi].protocol += "n";
+    }  else if (boost::regex_match(tl, sm, boost::regex(
+            "^.*?VHT Capabilities:", boost::regex_constants::icase))) { // protocol
+        pageBlock = BT_VHT_CAPABILITIES;
+        MainForm::cellDataRay[tbi].protocol += "ac";
+    } else if (boost::regex_match(tl, sm, boost::regex(".*?capability:.*?( ESS( .*|$))",
             boost::regex_constants::icase))) {
-        MainForm::cellDataRay[tbi].protocol = sm[1];
-//        cout << MainForm::cellDataRay[tbi].macAddr << "  " 
-//                << MainForm::cellDataRay[tbi].protocol << endl;
-    } else if (boost::regex_match(tl, sm, boost::regex(".*MODE:+ *([^ ]+).*",
+        MainForm::cellDataRay[tbi].mode = "AP"; // http://unix.stackexchange.com/questions/63069/the-n-mode-on-iwlist-wlan0-scan
+    } else if (boost::regex_match(tl, sm, boost::regex(".*?capability:.*?( IBSS( .*|$))",
             boost::regex_constants::icase))) {
-        MainForm::cellDataRay[tbi].mode = sm[1];
+        MainForm::cellDataRay[tbi].mode = "Ad Hoc";
     } else if (boost::regex_match(tl, sm,
-            boost::regex(" *Frequency *[:=] *([\\d\\.]+).*\\(Channel +(\\d{1,2})\\).*",
+          boost::regex("^.*?primary channel: +([0-9]+).*", boost::regex_constants::icase))) {
+        string tempChan = sm[1];
+        MainForm::cellDataRay[tbi].channel = atoi(tempChan.c_str());
+        MainForm::cellDataRay[tbi].cenChan = MainForm::cellDataRay[tbi].channel;
+    } else if (boost::regex_match(tl, sm,
+          boost::regex("^.*?DS Parameter set: +(channel)?.*?([0-9]+).*", boost::regex_constants::icase))
+          && MainForm::cellDataRay[tbi].channel == 0) {
+        string tempChan = sm[2];
+        MainForm::cellDataRay[tbi].channel = atoi(tempChan.c_str());
+        MainForm::cellDataRay[tbi].cenChan = MainForm::cellDataRay[tbi].channel;
+    } else if (boost::regex_match(tl, sm, boost::regex("^.*?secondary channel offset: *([^ \\t]+).*",
+          boost::regex_constants::icase))) { // secondary channel offset
+        string tempString = sm[1];
+        if (tempString == "above") MainForm::cellDataRay[tbi].cenChan = MainForm::cellDataRay[tbi].channel + 2;
+        else if (tempString == "below") MainForm::cellDataRay[tbi].cenChan = MainForm::cellDataRay[tbi].channel - 2;
+    } else if (boost::regex_match(tl, sm, 
+            boost::regex("^.*?freq:.*?([0-9]+).*",
             boost::regex_constants::icase))) {
         string tempFreq = sm[1];
         MainForm::cellDataRay[tbi].frequency = tempFreq;
-        string tempChan = sm[2];
-        MainForm::cellDataRay[tbi].channel = atoi(tempChan.c_str());
-        if (MainForm::cellDataRay[tbi].pBandCurve == 0) { // wish we knew the band at initial cell discovery
-            MainForm::cellDataRay[tbi].pBandCurve = new QwtPlotCurve("");
-            MainForm::cellDataRay[tbi].pBandCurve->setPen(* new QPen(MainForm::cellDataRay[tbi].color, 3.0));
-            MainForm::cellDataRay[tbi].pBandCurve->setRenderHint(QwtPlotItem::RenderAntialiased);
+        if (newBSS) {
             if (tempFreq.substr(0, 1) == "2") {
                 MainForm::cellDataRay[tbi].pBandCurve->attach(MainForm::mainFormWidget.chan24Plot);
             } else {
                 MainForm::cellDataRay[tbi].pBandCurve->attach(MainForm::mainFormWidget.chan5Plot);
             }
         }
-    /* fix for RTL8187SE oddball driver that doesn't want to report frequency */    
-    } else if (boost::regex_match(tl, sm, 
-            boost::regex(" *Channel *[:=] *(\\d+).*",
+    } else if (boost::regex_match(tl, sm, boost::regex("^.*?signal:.*?([\\-0-9]+).*?dBm.*",
             boost::regex_constants::icase))) {
-        string tempChan = sm[1];
-        int tempIChan = atoi(tempChan.c_str());
-        MainForm::cellDataRay[tbi].channel = tempIChan;
-        string tempFreq;
-        if (tempIChan > 0 && tempIChan <= 14 ) {
-            tempFreq = chan24Freq[tempIChan];
-        } else if (tempIChan > 14 && tempIChan <= 196) {
-            for (int iChan = 0; iChan < 42; iChan++) {
-                if (tempChan == chan50Freq[iChan][1]) {
-                    tempFreq == chan50Freq[iChan][2];
-                    break;
-                }
-            }
-        } else tempFreq = "0.000";
-        MainForm::cellDataRay[tbi].frequency = tempFreq;
-        if (MainForm::cellDataRay[tbi].pBandCurve == 0) {
-            MainForm::cellDataRay[tbi].pBandCurve = new QwtPlotCurve("");
-            MainForm::cellDataRay[tbi].pBandCurve->setPen(* new QPen(MainForm::cellDataRay[tbi].color, 3.0));
-            MainForm::cellDataRay[tbi].pBandCurve->setRenderHint(QwtPlotItem::RenderAntialiased);
-            if (tempFreq.substr(0, 1) == "2") {
-                MainForm::cellDataRay[tbi].pBandCurve->attach(MainForm::mainFormWidget.chan24Plot);
-            } else {
-                MainForm::cellDataRay[tbi].pBandCurve->attach(MainForm::mainFormWidget.chan5Plot);
-            }
-        } // end fix for idiotic RTL8187SE driver
-    } else if (boost::regex_match(tl, boost::regex(".*?Signal level[=:].*",
-            boost::regex_constants::icase))) {
-        // first try to get and normalize (0 to 100) a "quality" figure
-        boost::regex_match(tl, sm, boost::regex(".*?(\\d+)/(\\d+).*"));
-        string sNumer = sm[1];
-        string sDenom = sm[2];
-        double dNumer = atof(sNumer.c_str());
-        double dDenom = atof(sDenom.c_str());
-        if (dDenom == 0.0) dDenom = 100.0; // arbitrary fudge against divide by zero
-        MainForm::cellDataRay[tbi].quality = dNumer / dDenom * 100.0;
-        MainForm::cellDataRay[tbi].signal = cellDataRay[tbi].quality - 110;
-        // now, if we get a real signal value in dBm, use it
-        if (boost::regex_match(tl, sm, boost::regex(".*?Signal level *[:=] *([\\-0-9]+) *dBm.*",
-                boost::regex_constants::icase))) {
-            string sSig = sm[1];
-            MainForm::cellDataRay[tbi].signal = atoi(sSig.c_str());
-            MainForm::cellDataRay[tbi].quality = MainForm::cellDataRay[tbi].signal + 110;
-        }
-        if (MainForm::cellDataRay[tbi].signal < MainForm::cellDataRay[tbi].minSignal)
-            MainForm::cellDataRay[tbi].minSignal = MainForm::cellDataRay[tbi].signal;
-        if (MainForm::cellDataRay[tbi].signal > MainForm::cellDataRay[tbi].maxSignal)
-            MainForm::cellDataRay[tbi].maxSignal = MainForm::cellDataRay[tbi].signal;
-        // now, record our sample in the History. The following few lines need some reason why.
-        // Some flaky drivers report a MAC address more than once per scan. In some of
-        // these cases it seems that the first data is bogus. So, we overwrite the first
-        // history entry with the latest. Remember, we don't know that a second entry for
-        // a MAC might be coming...
-        if (MainForm::cellDataRay[tbi].timesSeen == 1) {
-            int ixTemp = MainForm::cellDataRay[tbi].pHistory->totalSamples % MAX_SAMPLES;
-            MainForm::cellDataRay[tbi].pHistory->sampleSec[ixTemp] = MainForm::blockSampleTime;
-            MainForm::cellDataRay[tbi].pHistory->sampleSec[ixTemp + MAX_SAMPLES]
+            string tempSig = sm[1];
+            MainForm::cellDataRay[tbi].signal = atoi(tempSig.c_str());
+            if (MainForm::cellDataRay[tbi].signal < MainForm::cellDataRay[tbi].minSignal)
+                MainForm::cellDataRay[tbi].minSignal = MainForm::cellDataRay[tbi].signal;
+            if (MainForm::cellDataRay[tbi].signal > MainForm::cellDataRay[tbi].maxSignal)
+                MainForm::cellDataRay[tbi].maxSignal = MainForm::cellDataRay[tbi].signal;
+            // add to history
+            if (MainForm::cellDataRay[tbi].timesSeen == 1) {
+                int ixTemp = MainForm::cellDataRay[tbi].pHistory->totalSamples % MAX_SAMPLES;
+                MainForm::cellDataRay[tbi].pHistory->sampleSec[ixTemp] = MainForm::blockSampleTime;
+                MainForm::cellDataRay[tbi].pHistory->sampleSec[ixTemp + MAX_SAMPLES]
                     = MainForm::blockSampleTime;
-            MainForm::cellDataRay[tbi].pHistory->signal[ixTemp] = MainForm::cellDataRay[tbi].signal;
-            MainForm::cellDataRay[tbi].pHistory->signal[ixTemp + MAX_SAMPLES] = MainForm::cellDataRay[tbi].signal;
-            MainForm::cellDataRay[tbi].pHistory->totalSamples++;
-        } else {
-            int ixTemp = (MainForm::cellDataRay[tbi].pHistory->totalSamples - 1) % MAX_SAMPLES;
-            MainForm::cellDataRay[tbi].pHistory->sampleSec[ixTemp] = MainForm::blockSampleTime;
-            MainForm::cellDataRay[tbi].pHistory->sampleSec[ixTemp + MAX_SAMPLES]
-                    = MainForm::blockSampleTime;
-            MainForm::cellDataRay[tbi].pHistory->signal[ixTemp] = MainForm::cellDataRay[tbi].signal;
-            MainForm::cellDataRay[tbi].pHistory->signal[ixTemp + MAX_SAMPLES] = MainForm::cellDataRay[tbi].signal;
-        }
-        // now, if we get a real noise value in dBm use it, otherwise fake it
-        if (boost::regex_match(tl, sm, boost::regex(".*?Noise level *[:=] *([\\-0-9]+) *dBm.*",
-                boost::regex_constants::icase))) {
-            string sNoise = sm[1];
-            MainForm::cellDataRay[tbi].noise = atoi(sNoise.c_str());
-        } else MainForm::cellDataRay[tbi].noise = -100; // fake it
-    } else if (boost::regex_match(tl, sm, boost::regex(".*?\\d{1,3} *Mb/s.*",
-            boost::regex_constants::icase))) { // scan thru all instances
-        string::const_iterator tlBegin = tl.begin(); // of "digits Mb/s" to find
-        string::const_iterator tlEnd = tl.end(); // the max.
-        boost::match_results<string::const_iterator> tlWhat;
-        while (boost::regex_search(tlBegin, tlEnd, tlWhat, boost::regex(".*?(\\d{1,3}) *Mb/s"))) {
-            int iWhat = atoi(string(tlWhat[1].first, tlWhat[2].second - 1).c_str());
-            if (iWhat > MainForm::cellDataRay[tbi].mbps) MainForm::cellDataRay[tbi].mbps = iWhat;
-            tlBegin = tlWhat[0].second;
-        }
-        // Here we deal with encryption privacy and ciphers.
-    } else if (boost::regex_match(tl, sm, boost::regex(" *Encryption key: *([^ ]+) *",
-            boost::regex_constants::icase))) {
-        MainForm::cellDataRay[tbi].security = sm[1];
-    } else if (boost::regex_match(tl, sm, boost::regex(" *IE: *WPA .*",
-            boost::regex_constants::icase))) {
-        currentPrivacy = WEP;
-        if (MainForm::cellDataRay[tbi].privacy == none) {
-            MainForm::cellDataRay[tbi].privacy = WEP;
-        }
-    } else if (boost::regex_match(tl, sm, boost::regex(" *IE: .+?WPA2 .*",
-            boost::regex_constants::icase))) {
-        currentPrivacy = WPA2;
-        MainForm::cellDataRay[tbi].privacy = WPA2;
-    } else if (boost::regex_match(tl, sm, boost::regex(" *Group Cipher *: *([^ ]+).*",
-            boost::regex_constants::icase))) {
-        if (currentPrivacy == MainForm::cellDataRay[tbi].privacy) {
-            if (sm[1] == "TKIP") {
-                MainForm::cellDataRay[tbi].cipher = TKIP;
-            } else if (sm[1] == "CCMP") {
-                MainForm::cellDataRay[tbi].cipher = AES;
+                MainForm::cellDataRay[tbi].pHistory->signal[ixTemp] = MainForm::cellDataRay[tbi].signal;
+                MainForm::cellDataRay[tbi].pHistory->signal[ixTemp + MAX_SAMPLES] = MainForm::cellDataRay[tbi].signal;
+                MainForm::cellDataRay[tbi].pHistory->totalSamples++;
             } else {
-
-                MainForm::cellDataRay[tbi].cipher = unknown;
+                int ixTemp = (MainForm::cellDataRay[tbi].pHistory->totalSamples - 1) % MAX_SAMPLES;
+                MainForm::cellDataRay[tbi].pHistory->sampleSec[ixTemp] = MainForm::blockSampleTime;
+                MainForm::cellDataRay[tbi].pHistory->sampleSec[ixTemp + MAX_SAMPLES]
+                    = MainForm::blockSampleTime;
+                MainForm::cellDataRay[tbi].pHistory->signal[ixTemp] = MainForm::cellDataRay[tbi].signal;
+                MainForm::cellDataRay[tbi].pHistory->signal[ixTemp + MAX_SAMPLES] = MainForm::cellDataRay[tbi].signal;
             }
-        }
+            if (MainForm::cellDataRay[tbi].signal <= -100) MainForm::cellDataRay[tbi].quality = 0;
+            else if(MainForm::cellDataRay[tbi].signal >= -50) MainForm::cellDataRay[tbi].quality = 100;
+            else MainForm::cellDataRay[tbi].quality = 2 * (MainForm::cellDataRay[tbi].signal + 100);
+    } else if (boost::regex_match(tl, sm, boost::regex(".*?Group cipher: *(.*)",
+            boost::regex_constants::icase))) { // group cipher
+        MainForm::cellDataRay[tbi].privacy = sm[1];
+    } else if (pageBlock == BT_HT_CAPABILITIES && boost::regex_match(tl, sm, boost::regex(".*?HT20/HT40.*",
+            boost::regex_constants::icase))) { // Bandwidth HT 40 MHz
+        MainForm::cellDataRay[tbi].BW = 40;
+    } else if (pageBlock == BT_VHT_OPERATION && boost::regex_match(tl, sm, boost::regex(".*?\\* channel width:.*?([0-9]+) MHz.*",
+            boost::regex_constants::icase))) { // Bandwidth VHT
+        string tempString = sm[1];
+        MainForm::cellDataRay[tbi].BW = atoi(tempString.c_str());
+    } else if (boost::regex_match(tl, sm, boost::regex(".*?Pairwise ciphers: *(.*)",
+            boost::regex_constants::icase))) { // pairwise ciphers
+        MainForm::cellDataRay[tbi].cipher = sm[1];
+    } else if (boost::regex_match(tl, sm, boost::regex(".*?Authentication suites: *(.*)",
+            boost::regex_constants::icase))) { // authentication
+            MainForm::cellDataRay[tbi].security = sm[1];
+    } else if (boost::regex_match(tl, sm, boost::regex(".*?RSN: *(.*)",
+            boost::regex_constants::icase))) { pageBlock = BT_RSN;
+    } else if (boost::regex_match(tl, sm, boost::regex(".*?BSS Load: *(.*)",
+            boost::regex_constants::icase))) { pageBlock = BT_BSS_LOAD;
+    } else if (boost::regex_match(tl, sm, boost::regex("[^V]*HT operation: *(.*)",
+            boost::regex_constants::icase))) { pageBlock = BT_HT_OPERATION;
+    } else if (boost::regex_match(tl, sm, boost::regex(".*?Extended capabilities: *(.*)",
+            boost::regex_constants::icase))) { pageBlock = BT_EXTENDED_CAPABILITIES;
+    } else if (boost::regex_match(tl, sm, boost::regex(".*?VHT operation:.*",
+            boost::regex_constants::icase))) { pageBlock = BT_VHT_OPERATION;
+    } else if (boost::regex_match(tl, sm, boost::regex(".*?WPA: *(.*)",
+            boost::regex_constants::icase))) { pageBlock = BT_WPA;
+    } else if (boost::regex_match(tl, sm, boost::regex(".*?WPS: *(.*)",
+            boost::regex_constants::icase))) { pageBlock = BT_WPS;
+    } else if (boost::regex_match(tl, sm, boost::regex(".*?WMM: *(.*)",
+            boost::regex_constants::icase))) { pageBlock = BT_WMM;
     }
-}
+} // extractData()
 
 void MainForm::doLogData() {
     if (!MainForm::logDataStream.is_open()) { // if data log file is not open...
@@ -1263,15 +1238,15 @@ void MainForm::doLogData() {
                 << MainForm::cellDataRay[tbi].mode << "\t"
                 << MainForm::cellDataRay[tbi].protocol << "\t"
                 << MainForm::cellDataRay[tbi].security << "\t"
-                << MainForm::wlPrivacies[MainForm::cellDataRay[tbi].privacy] << "\t"
-                << MainForm::wlCiphers[MainForm::cellDataRay[tbi].cipher] << "\t"
+                << MainForm::cellDataRay[tbi].privacy << "\t"
+                << MainForm::cellDataRay[tbi].cipher << "\t"
                 << MainForm::cellDataRay[tbi].frequency << "\t"
                 << MainForm::cellDataRay[tbi].quality << "\t"
                 << MainForm::cellDataRay[tbi].signal << "\t"
-                << MainForm::cellDataRay[tbi].noise << "\t"
+                << MainForm::cellDataRay[tbi].BW << "\t"
                 << MainForm::cellDataRay[tbi].minSignal << "\t"
                 << MainForm::cellDataRay[tbi].maxSignal << "\t"
-                << MainForm::cellDataRay[tbi].mbps << "\t"
+                << MainForm::cellDataRay[tbi].cenChan << "\t"
                 << firstStr << "\t"
                 << lastStr << "\t"
                 << "\"" << MainForm::cellDataRay[tbi].vendor << "\"\n";
@@ -1283,7 +1258,7 @@ void MainForm::writeLogDataHeader(string logDataFileName) {
     MainForm::logDataStream.open(logDataFileName, ios::out | ios::app); // open for write append
     MainForm::logDataStream << "LINSSIDDATALOGVER " << LINSSIDDATALOGVER << "\n";
     MainForm::logDataStream << "Time\tSSID\tMAC\tChannel\tMode\tProtocol\tSecurity\tPrivacy\t\
-Cipher\tFrequency\tQuality\tSignal\tNoise\tMin_Sig\tMax_Sig\tMbps\tFirst_Seen\tLast_Seen\tVendor\n";
+Cipher\tFrequency\tQuality\tSignal\tBW\tMin_Sig\tMax_Sig\tCen_Chan\tFirst_Seen\tLast_Seen\tVendor\n";
 }
 
 void MainForm::handleDataReadyEvent(const DataReadyEvent * /*event*/) {
@@ -1294,10 +1269,10 @@ void MainForm::handleDataReadyEvent(const DataReadyEvent * /*event*/) {
     static fstream thePipe(pipeName);
     // reset some stuff from last pass
     for (int ix = 0; ix <= maxTableIndex; ix++) {
-        MainForm::cellDataRay[ix].quality = 0;
+        MainForm::cellDataRay[ix].protocol = "";
         MainForm::cellDataRay[ix].signal = -110;
-        MainForm::cellDataRay[ix].noise = -110;
         MainForm::cellDataRay[ix].timesSeen = 0;
+        // MainForm::cellDataRay[ix].BW = 20; // no bandwidth less than 20 MHz
     }
     // disable column sorting to prevent segfaults while we muck with the table
     MainForm::mainFormWidget.mainTableWidget->setSortingEnabled(false);
@@ -1305,11 +1280,9 @@ void MainForm::handleDataReadyEvent(const DataReadyEvent * /*event*/) {
     bool lastLine = false;
     MainForm::now = time(NULL);
     MainForm::blockSampleTime = now - MainForm::runStartTime;
-    currentPrivacy = none;
     int tableIndex = -1; // holds current index pointer into cellData
     while (!lastLine && getline(thePipe, tempLine)) {
-        //        getline(thePipe, tempLine);
-//        cout << "tempLine:" << tempLine << endl;
+        // cout << "tempLine:" << tempLine << endl; //debug
         MainForm::extractData(tempLine, tableIndex); // the heavy lifting here
         lastLine = tempLine.substr(0, endBlockString.length()) == endBlockString;
         if (lastLine) {
@@ -1335,4 +1308,30 @@ void MainForm::handleDataReadyEvent(const DataReadyEvent * /*event*/) {
 inline void MainForm::waste(int) {
     // This silliness is to ignore an argument function's return code without
     // having the compiler whine about it.
-}
\ No newline at end of file
+}
+
+int MainForm::MaxIntStr(const string &s) {
+    stringstream ss(s);
+    string item;
+    int retInt = INT_MIN;
+    int tempInt;
+    while (getline(ss, item, ' ')) {
+        if (item.back() == '*') item=item.substr(0,item.length()-1);
+        tempInt = atoi(item.c_str());
+        if (tempInt > retInt) retInt = tempInt;
+    }
+    return retInt;
+}
+
+int MainForm::MinIntStr(const string &s) {
+    stringstream ss(s);
+    string item;
+    int retInt = INT_MAX;
+    int tempInt;
+    while (getline(ss, item, ' ')) {
+        if (item.back() == '*') item=item.substr(0,item.length()-1);
+        tempInt = atoi(item.c_str());
+        if (tempInt < retInt) retInt = tempInt;
+    }
+    return retInt;
+}
